==> soliq_id-1.4.27.tar.gz: lib/src/screens/passive_camera_screen2.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math' as math;
import 'dart:ui';

import 'package:camera/camera.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:google_mlkit_face_detection/google_mlkit_face_detection.dart';
import 'package:image/image.dart' as img;
import 'package:soliq_id/localizations_util.dart';
import 'package:soliq_id/src/utils/navigator_extension.dart';

import '../../soliq_id.dart';
import '../data/model/angle_model.dart';
import '../utils/color_utils.dart';
import '../utils/faceid_localizations.dart';
import '../utils/progress_indicator/src/circular_step_progress_indicator.dart';
import '../utils/utils.dart';

class PassiveCameraScreen extends StatefulWidget {
  const PassiveCameraScreen({super.key,});

  @override
  State<PassiveCameraScreen> createState() => _PassiveCameraScreenState();
}

class _PassiveCameraScreenState extends State<PassiveCameraScreen> {
  final FaceDetector _faceDetector = FaceDetector(
    options: FaceDetectorOptions(
      enableContours: false,
      enableClassification: true,
      enableLandmarks: false,
      performanceMode: FaceDetectorMode.fast,
    ),
  );
  CameraController? _controller;
  int _cameraIndex = -1;
  var faceCount = -1;

  bool _canProcess = true, _isBusy = false, _isValid = false;

  Rect? boundingBox;
  double _widthFace = 0.0, _heightFace = 0.0;
  String hint = "";

  Timer? _timer;
  int counter = 3;
  bool isTakePictureProcess = false;
  double zoomLevel = 0.0, minZoomLevel = 0.0, maxZoomLevel = 0.0;

  @override
  void initState() {
    super.initState();

    if (cameras.any(
          (element) =>
      element.lensDirection == CameraLensDirection.front &&
          element.sensorOrientation == 90,
    )) {
      _cameraIndex = cameras.indexOf(
        cameras.firstWhere((element) =>
        element.lensDirection == CameraLensDirection.front &&
            element.sensorOrientation == 90),
      );
    } else {
      for (var i = 0; i < cameras.length; i++) {
        if (cameras[i].lensDirection == CameraLensDirection.front) {
          _cameraIndex = i;
          break;
        }
      }
    }

    if (_cameraIndex != -1) {
      _startLiveFeed();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: AppColor.appColor,
      appBar: AppBar(
        elevation: 0,
        centerTitle: true,
        title: const Text(
          "YuzID",
          style: TextStyle(
              fontSize: 16, fontWeight: FontWeight.w500, color: Colors.black),
        ),
        backgroundColor: AppColor.appColor,
      ),
      body: SingleChildScrollView(
        child: Column(
          children: [
            SizedBox(
              child: Stack(
                alignment: AlignmentDirectional.center,
                children: [
                  _liveFeedBody(),
                ],
              ),
            ),
            SizedBox(
              height: 10,
            ),
            _isValid ? Text(
              " $counter",
              style: TextStyle(
                color:   Colors.black,
                fontSize: 40,

              ),
            ) : SizedBox(),
            const SizedBox(
              height: 20,
            ),
            Container(
              padding: const EdgeInsets.all(12),
              margin: const EdgeInsets.all(12),
              decoration: BoxDecoration(
                color: faceCount != 1
                    ? AppColor.custom_redColor.withOpacity(
                  0.2,
                )
                    : const Color(0xFFF3F6F5),
                borderRadius: BorderRadius.circular(
                  12,
                ),
              ),
              child: Column(
                children: [
                  Container(
                    margin: const EdgeInsets.symmetric(
                      horizontal: 8,
                    ),
                    padding: const EdgeInsets.symmetric(
                      horizontal: 40,
                      vertical: 16,
                    ),
                    decoration: BoxDecoration(
                      color: _isValid ? Colors.white : Colors.red,
                      borderRadius: const BorderRadius.all(
                        Radius.circular(16),
                      ),
                    ),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        const SizedBox(
                          width: 8,
                        ),
                        Text(
                          getString( hint),
                          style: TextStyle(
                            color: !_isValid ? Colors.white : Colors.black,
                          ),
                        ),
                        const SizedBox(
                          width: 8,
                        )
                      ],
                    ),
                  ),
                  const SizedBox(
                    height: 18,
                  ),
                  // faceCountDetection(),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  Widget _liveFeedBody() {
    const double previewAspectRatio = 1;
    if (_controller?.value.isInitialized == false) {
      return  Container(
        alignment: Alignment.topCenter,
        margin: const EdgeInsets.only(
          left: 16,
          right: 16,
        ),
        child: AspectRatio(
          aspectRatio: 1.0 / previewAspectRatio,
          child: CircularStepProgressIndicator(
            totalSteps: 800,
            currentStep: 800,
            stepSize: 4,
            selectedColor: _isValid ? AppColor.chartActiveColor : Colors.red,
            // unselectedColor: Colors.grey.withOpacity(0.2),
            padding: math.pi / 35,
            child: Stack(
              alignment: Alignment.center,
              children: [
                Container(
                  margin: const EdgeInsets.all(4),
                  child: ClipRRect(
                      borderRadius: BorderRadius.all(
                          Radius.circular(MediaQuery.of(context).size.width / 2)),
                      child: Container(
                        color: Colors.black,
                      )
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    }
    return Container(
      alignment: Alignment.topCenter,
      margin: const EdgeInsets.only(
        left: 16,
        right: 16,
      ),
      child: AspectRatio(
        aspectRatio: 1.0 / previewAspectRatio,
        child: CircularStepProgressIndicator(
          totalSteps: 800,
          currentStep: 800,
          stepSize: 4,
          selectedColor: _isValid ? AppColor.chartActiveColor : Colors.red,
          // unselectedColor: Colors.grey.withOpacity(0.2),
          padding: math.pi / 35,
          child: Stack(
            alignment: Alignment.center,
            children: [
              Container(
                margin: const EdgeInsets.all(4),
                child: ClipRRect(
                  borderRadius: BorderRadius.all(
                      Radius.circular(MediaQuery.of(context).size.width / 2)),
                  child: Transform.scale(
                    scale: _controller!.value.aspectRatio / previewAspectRatio,
                    child: Stack(
                      children: [
                        Center(
                          child: CameraPreview(_controller!),
                        ),
                        Visibility(
                          visible: false, //widget.value >= 96,
                          child: Stack(
                            children: [
                              BackdropFilter(
                                filter: ImageFilter.blur(sigmaX: 5, sigmaY: 5),
                                child: const SizedBox(),
                              ),
                            ],
                          ),
                        )
                      ],
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Future _startLiveFeed() async {
    final camera = cameras[_cameraIndex];
    _controller = CameraController(
      camera,
      ResolutionPreset.high,
      enableAudio: false,
    );

    try {
      _controller?.initialize().then((_) {
        if (!mounted) {
          return;
        }
        _controller?.getMinZoomLevel().then((value) {
          zoomLevel = value;
          minZoomLevel = value;
        });
        _controller?.getMaxZoomLevel().then((value) {
          maxZoomLevel = value;
        });
        _controller?.startImageStream(_processCameraImage);
        setState(() {});
      });
    } catch (error) {
      Utils.onError(context, error);
    }
  }

  Future _processCameraImage(CameraImage image) async {
    final WriteBuffer allBytes = WriteBuffer();
    for (final Plane plane in image.planes) {
      allBytes.putUint8List(plane.bytes);
    }
    final bytes = allBytes.done().buffer;

    final Size imageSize =
    Size(image.width.toDouble(), image.height.toDouble());

    final camera = cameras[_cameraIndex];
    final imageRotation =
    InputImageRotationValue.fromRawValue(camera.sensorOrientation);
    if (imageRotation == null) return;

    final inputImageFormat =
    InputImageFormatValue.fromRawValue(image.format.raw);
    if (inputImageFormat == null) return;

    final planeData = image.planes.map(
          (Plane plane) {
        return InputImagePlaneMetadata(
          bytesPerRow: plane.bytesPerRow,
          height: plane.height,
          width: plane.width,
        );
      },
    ).toList();

    final inputImageData = InputImageData(
      size: imageSize,
      imageRotation: imageRotation,
      inputImageFormat: inputImageFormat,
      planeData: planeData,
    );

    final inputImage = InputImage.fromBytes(
        bytes: bytes.asUint8List(), inputImageData: inputImageData);
    await processImage(inputImage);
  }

  Future<void> processImage(
      InputImage inputImage,
      ) async {
    if (isTakePictureProcess) return;
    if (!_canProcess) return;
    if (_isBusy) return;
    _isBusy = true;
    final faces = await _faceDetector.processImage(inputImage);
    if (inputImage.inputImageData?.size != null &&
        inputImage.inputImageData?.imageRotation != null) {
      faceCount = faces.length;
      var isValid = false;
      if (faces.length > 1) {
        hint = "faces_are";
      } else if (faces.length == 1) {
        final face = faces.first;

        boundingBox = face.boundingBox;
        _heightFace = boundingBox!.right - boundingBox!.left;
        _widthFace = boundingBox!.bottom - boundingBox!.top;

        var rightStateFace = isFaceRightState(face);

        if (AngleModel.MinFaceSize <= _heightFace ||
            AngleModel.MinFaceSize <= _widthFace) {
          hint = "face_away";
        }
        if (AngleModel.MaxFaceSize >= _heightFace ||
            AngleModel.MaxFaceSize >= _widthFace) {
          hint = "face_closer";
        }
        var maxRange = 7;
        if ((face.headEulerAngleX ?? 0) <= -1 * maxRange) {
          hint = "head_rise";
        } else if ((face.headEulerAngleX ?? 0) >= maxRange) {
          hint = "head_down";
        } else if ((face.headEulerAngleY ?? 0) >= maxRange) {
          hint = Platform.isAndroid ? "head_right" : "head_left";
        } else if ((face.headEulerAngleY ?? 0) <= -1 * maxRange) {
          hint = Platform.isAndroid ? "head_left" : "head_right";
        } else if ((face.headEulerAngleZ ?? 0) <= -1 * maxRange ||
            (face.headEulerAngleZ ?? 0) >= maxRange) {
          hint = "head_straight";
        } else if ((face.leftEyeOpenProbability ?? 0) <= 0.8 ||
            (face.rightEyeOpenProbability ?? 0) <= 0.8) {
          hint = "open_eyes";
        }  else if (rightStateFace ) {
          hint = "straight";
          isValid = true;
        }
      } else {
        hint = "no_face_found";
      }
      _isValid = isValid;
    } else {
      _isValid = false;
    }
    if (_isValid) {
      if (!isTakePictureProcess) startTimer();
    } else {
      stopTimer();
    }
    _isBusy = false;
    setState(() {});
  }

  bool isFaceRightState(Face face) {
    boundingBox = face.boundingBox;
    _heightFace = boundingBox!.right - boundingBox!.left;
    _widthFace = boundingBox!.bottom - boundingBox!.top;
    return AngleModel.MinFaceSize <= _heightFace &&
        _heightFace <= AngleModel.MaxFaceSize &&
        AngleModel.MinFaceSize <= _widthFace &&
        _widthFace <= AngleModel.MaxFaceSize && (face.headEulerAngleY ?? 0) >= -8 &&
        (face.headEulerAngleY ?? 0) <= 12 &&
        (face.headEulerAngleX ?? 0) >= -5 &&
        (face.headEulerAngleX ?? 0) <= 10 &&
        (face.headEulerAngleZ ?? 0) >= -5 &&
        (face.headEulerAngleZ ?? 0) <= 5 &&
        (face.leftEyeOpenProbability ?? 0) >= 0.8 &&
        (face.rightEyeOpenProbability ?? 0) >= 0.8 ;
  }

  void _onTimerTick(Timer timer) async {
    counter--;
    if (counter == 0) {
      isTakePictureProcess = true;
      final xFile = await _controller?.takePicture();
      if (xFile == null) {
        isTakePictureProcess = false;
        stopTimer();
      } else {
        final inputImage = InputImage.fromFile(File(xFile.path));
        final faces = await _faceDetector.processImage(inputImage);
        if (faces.length == 1) {
          if (isFaceRightState(faces.first)) {
            final bytes = await xFile.readAsBytes();
            img.Image originalImage =
                img.decodeImage(bytes) ?? img.Image(width: 0, height: 0);
            var scale = calcScale(
              srcWidth: originalImage.width * 1.0,
              srcHeight: originalImage.height * 1.0,
              minWidth: 480,
              minHeight: 640,
            );
            final imageUintList = await FlutterImageCompress.compressWithList(
              bytes,
              minWidth: originalImage.width ~/ scale,
              minHeight: originalImage.height ~/ scale,
            );
            stopTimer();
            pop(result: base64Encode(imageUintList));
            // widget.onEvent.call(base64Encode(imageUintList));
          } else {
            isTakePictureProcess = false;
            stopTimer();
          }
        } else {
          isTakePictureProcess = false;
          stopTimer();
        }
      }
    }
    if (!mounted) setState(() {});
  }

  void startTimer() {
    if (_timer?.isActive == false || _timer == null) {
      _timer = Timer.periodic(const Duration(seconds: 1), _onTimerTick);
    }
  }

  void stopTimer() {
    if (_timer?.isActive == true) {
      _timer?.cancel();
      counter = 3;
    }
  }

  @override
  void dispose() {
    _stopLiveFeed();
    super.dispose();
  }

  Future _stopLiveFeed() async {
    await _controller?.stopImageStream();
    await _controller?.dispose();
    _controller = null;
  }
}

==> soliq_id-1.4.33.tar.gz: lib/src/screens/passive_camera_screen2.dart <==
import 'dart:async';
import 'dart:convert';
import 'dart:io';
import 'dart:math' as math;
import 'dart:ui';

import 'package:camera/camera.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_image_compress/flutter_image_compress.dart';
import 'package:google_mlkit_face_detection/google_mlkit_face_detection.dart';
import 'package:image/image.dart' as img;
import 'package:soliq_id/localizations_util.dart';
import 'package:soliq_id/src/utils/navigator_extension.dart';

import '../../soliq_id.dart';
import '../data/model/angle_model.dart';
import '../utils/color_utils.dart';
import '../utils/faceid_localizations.dart';
import '../utils/progress_indicator/src/circular_step_progress_indicator.dart';
import '../utils/utils.dart';

class PassiveCameraScreen extends StatefulWidget {
  const PassiveCameraScreen({super.key,});

  @override
  State<PassiveCameraScreen> createState() => _PassiveCameraScreenState();
}

class _PassiveCameraScreenState extends State<PassiveCameraScreen> {
  final FaceDetector _faceDetector = FaceDetector(
    options: FaceDetectorOptions(
      enableContours: false,
      enableClassification: true,
      enableLandmarks: false,
      performanceMode: FaceDetectorMode.fast,
    ),
  );
  CameraController? _controller;
  int _cameraIndex = -1;
  var faceCount = -1;

  bool _canProcess = true, _isBusy = false, _isValid = false;

  Rect? boundingBox;
  double _widthFace = 0.0, _heightFace = 0.0;
  String hint = "";

  Timer? _timer;
  int counter = 3;
  bool isTakePictureProcess = false;
  double zoomLevel = 0.0, minZoomLevel = 0.0, maxZoomLevel = 0.0;

  @override
  void initState() {
    super.initState();

    if (cameras.any(
          (element) =>
      element.lensDirection == CameraLensDirection.front &&
          element.sensorOrientation == 90,
    )) {
      _cameraIndex = cameras.indexOf(
        cameras.firstWhere((element) =>
        element.lensDirection == CameraLensDirection.front &&
            element.sensorOrientation == 90),
      );
    } else {
      for (var i = 0; i < cameras.length; i++) {
        if (cameras[i].lensDirection == CameraLensDirection.front) {
          _cameraIndex = i;
          break;
        }
      }
    }

    if (_cameraIndex != -1) {
      _startLiveFeed();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: AppColor.appColor,
      appBar: AppBar(
        elevation: 0,
        centerTitle: true,
        title: const Text(
          "YuzID",
          style: TextStyle(
              fontSize: 16, fontWeight: FontWeight.w500, color: Colors.black),
        ),
        backgroundColor: AppColor.appColor,
      ),
      body: SingleChildScrollView(
        child: Column(
          children: [
            SizedBox(
              child: Stack(
                alignment: AlignmentDirectional.center,
                children: [
                  _liveFeedBody(),
                ],
              ),
            ),
            SizedBox(
              height: 10,
            ),
            _isValid ? Text(
              " $counter",
              style: TextStyle(
                color:   Colors.black,
                fontSize: 40,

              ),
            ) : SizedBox(),
            const SizedBox(
              height: 20,
            ),
            Container(
              padding: const EdgeInsets.all(12),
              margin: const EdgeInsets.all(12),
              decoration: BoxDecoration(
                color: faceCount != 1
                    ? AppColor.custom_redColor.withOpacity(
                  0.2,
                )
                    : const Color(0xFFF3F6F5),
                borderRadius: BorderRadius.circular(
                  12,
                ),
              ),
              child: Column(
                children: [
                  Container(
                    margin: const EdgeInsets.symmetric(
                      horizontal: 8,
                    ),
                    padding: const EdgeInsets.symmetric(
                      horizontal: 40,
                      vertical: 16,
                    ),
                    decoration: BoxDecoration(
                      color: _isValid ? Colors.white : Colors.red,
                      borderRadius: const BorderRadius.all(
                        Radius.circular(16),
                      ),
                    ),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        const SizedBox(
                          width: 8,
                        ),
                        Text(
                          getString( hint),
                          style: TextStyle(
                            color: !_isValid ? Colors.white : Colors.black,
                          ),
                        ),
                        const SizedBox(
                          width: 8,
                        )
                      ],
                    ),
                  ),
                  const SizedBox(
                    height: 18,
                  ),
                  // faceCountDetection(),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  Widget _liveFeedBody() {
    const double previewAspectRatio = 1;
    if (_controller?.value.isInitialized == false) {
      return  Container(
        alignment: Alignment.topCenter,
        margin: const EdgeInsets.only(
          left: 16,
          right: 16,
        ),
        child: AspectRatio(
          aspectRatio: 1.0 / previewAspectRatio,
          child: CircularStepProgressIndicator(
            totalSteps: 800,
            currentStep: 800,
            stepSize: 4,
            selectedColor: _isValid ? AppColor.chartActiveColor : Colors.red,
            // unselectedColor: Colors.grey.withOpacity(0.2),
            padding: math.pi / 35,
            child: Stack(
              alignment: Alignment.center,
              children: [
                Container(
                  margin: const EdgeInsets.all(4),
                  child: ClipRRect(
                      borderRadius: BorderRadius.all(
                          Radius.circular(MediaQuery.of(context).size.width / 2)),
                      child: Container(
                        color: Colors.black,
                      )
                  ),
                ),
              ],
            ),
          ),
        ),
      );
    }
    return Container(
      alignment: Alignment.topCenter,
      margin: const EdgeInsets.only(
        left: 16,
        right: 16,
      ),
      child: AspectRatio(
        aspectRatio: 1.0 / previewAspectRatio,
        child: CircularStepProgressIndicator(
          totalSteps: 800,
          currentStep: 800,
          stepSize: 4,
          selectedColor: _isValid ? AppColor.chartActiveColor : Colors.red,
          // unselectedColor: Colors.grey.withOpacity(0.2),
          padding: math.pi / 35,
          child: Stack(
            alignment: Alignment.center,
            children: [
              Container(
                margin: const EdgeInsets.all(4),
                child: ClipRRect(
                  borderRadius: BorderRadius.all(
                      Radius.circular(MediaQuery.of(context).size.width / 2)),
                  child: Transform.scale(
                    scale: _controller!.value.aspectRatio / previewAspectRatio,
                    child: Stack(
                      children: [
                        Center(
                          child: CameraPreview(_controller!),
                        ),
                        Visibility(
                          visible: false, //widget.value >= 96,
                          child: Stack(
                            children: [
                              BackdropFilter(
                                filter: ImageFilter.blur(sigmaX: 5, sigmaY: 5),
                                child: const SizedBox(),
                              ),
                            ],
                          ),
                        )
                      ],
                    ),
                  ),
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }

  Future _startLiveFeed() async {
    final camera = cameras[_cameraIndex];
    _controller = CameraController(
      camera,
      ResolutionPreset.high,
      enableAudio: false,
    );

    try {
      _controller?.initialize().then((_) {
        if (!mounted) {
          return;
        }
        _controller?.getMinZoomLevel().then((value) {
          zoomLevel = value;
          minZoomLevel = value;
        });
        _controller?.getMaxZoomLevel().then((value) {
          maxZoomLevel = value;
        });
        _controller?.startImageStream(_processCameraImage);
        setState(() {});
      });
    } catch (error) {
      Utils.onError(context, error);
    }
  }

  Future _processCameraImage(CameraImage image) async {
    final WriteBuffer allBytes = WriteBuffer();
    for (final Plane plane in image.planes) {
      allBytes.putUint8List(plane.bytes);
    }
    final bytes = allBytes.done().buffer;

    final Size imageSize =
    Size(image.width.toDouble(), image.height.toDouble());

    final camera = cameras[_cameraIndex];
    final imageRotation =
    InputImageRotationValue.fromRawValue(camera.sensorOrientation);
    if (imageRotation == null) return;

    final inputImageFormat =
    InputImageFormatValue.fromRawValue(image.format.raw);
    if (inputImageFormat == null) return;
    //
    // final planeData = image.planes.map(
    //       (Plane plane) {
    //     return InputImagePlaneMetadata(
    //       bytesPerRow: plane.bytesPerRow,
    //       height: plane.height,
    //       width: plane.width,
    //     );
    //   },
    // ).toList();
    //
    // final inputImageData = InputImageData(
    //   size: imageSize,
    //   imageRotation: imageRotation,
    //   inputImageFormat: inputImageFormat,
    //   planeData: planeData,
    // );
    //
    // final inputImage = InputImage.fromBytes(
    //     bytes: bytes.asUint8List(), inputImageData: inputImageData);
    // if (image.planes.length != 1) return null;
    final plane = image.planes.first;

    final inputImage = InputImage.fromBytes(
        bytes: bytes.asUint8List(),
        metadata: InputImageMetadata(
            size: Size(image.width.toDouble(), image.height.toDouble()),
            rotation: imageRotation,
            format: inputImageFormat,
            bytesPerRow: plane.bytesPerRow));
    await processImage(inputImage);
  }

  Future<void> processImage(
      InputImage inputImage,
      ) async {
    if (isTakePictureProcess) return;
    if (!_canProcess) return;
    if (_isBusy) return;
    _isBusy = true;
    final faces = await _faceDetector.processImage(inputImage);
    if (inputImage.metadata?.size != null &&
        inputImage.metadata?.rotation != null) {
      faceCount = faces.length;
      var isValid = false;
      if (faces.length > 1) {
        hint = "faces_are";
      } else if (faces.length == 1) {
        final face = faces.first;

        boundingBox = face.boundingBox;
        _heightFace = boundingBox!.right - boundingBox!.left;
        _widthFace = boundingBox!.bottom - boundingBox!.top;

        var rightStateFace = isFaceRightState(face);

        if (AngleModel.MinFaceSize <= _heightFace ||
            AngleModel.MinFaceSize <= _widthFace) {
          hint = "face_away";
        }
        if (AngleModel.MaxFaceSize >= _heightFace ||
            AngleModel.MaxFaceSize >= _widthFace) {
          hint = "face_closer";
        }
        var maxRange = 7;
        if ((face.headEulerAngleX ?? 0) <= -1 * maxRange) {
          hint = "head_rise";
        } else if ((face.headEulerAngleX ?? 0) >= maxRange) {
          hint = "head_down";
        } else if ((face.headEulerAngleY ?? 0) >= maxRange) {
          hint = Platform.isAndroid ? "head_right" : "head_left";
        } else if ((face.headEulerAngleY ?? 0) <= -1 * maxRange) {
          hint = Platform.isAndroid ? "head_left" : "head_right";
        } else if ((face.headEulerAngleZ ?? 0) <= -1 * maxRange ||
            (face.headEulerAngleZ ?? 0) >= maxRange) {
          hint = "head_straight";
        } else if ((face.leftEyeOpenProbability ?? 0) <= 0.8 ||
            (face.rightEyeOpenProbability ?? 0) <= 0.8) {
          hint = "open_eyes";
        }  else if (rightStateFace ) {
          hint = "straight";
          isValid = true;
        }
      } else {
        hint = "no_face_found";
      }
      _isValid = isValid;
    } else {
      _isValid = false;
    }
    if (_isValid) {
      if (!isTakePictureProcess) startTimer();
    } else {
      stopTimer();
    }
    _isBusy = false;
    setState(() {});
  }

  bool isFaceRightState(Face face) {
    boundingBox = face.boundingBox;
    _heightFace = boundingBox!.right - boundingBox!.left;
    _widthFace = boundingBox!.bottom - boundingBox!.top;
    return AngleModel.MinFaceSize <= _heightFace &&
        _heightFace <= AngleModel.MaxFaceSize &&
        AngleModel.MinFaceSize <= _widthFace &&
        _widthFace <= AngleModel.MaxFaceSize && (face.headEulerAngleY ?? 0) >= -8 &&
        (face.headEulerAngleY ?? 0) <= 12 &&
        (face.headEulerAngleX ?? 0) >= -5 &&
        (face.headEulerAngleX ?? 0) <= 10 &&
        (face.headEulerAngleZ ?? 0) >= -5 &&
        (face.headEulerAngleZ ?? 0) <= 5 &&
        (face.leftEyeOpenProbability ?? 0) >= 0.8 &&
        (face.rightEyeOpenProbability ?? 0) >= 0.8 ;
  }

  void _onTimerTick(Timer timer) async {
    counter--;
    if (counter == 0) {
      isTakePictureProcess = true;
      final xFile = await _controller?.takePicture();
      if (xFile == null) {
        isTakePictureProcess = false;
        stopTimer();
      } else {
        final inputImage = InputImage.fromFile(File(xFile.path));
        final faces = await _faceDetector.processImage(inputImage);
        if (faces.length == 1) {
          if (isFaceRightState(faces.first)) {
            final bytes = await xFile.readAsBytes();
            img.Image originalImage =
                img.decodeImage(bytes) ?? img.Image(width: 0, height: 0);
            var scale = calcScale(
              srcWidth: originalImage.width * 1.0,
              srcHeight: originalImage.height * 1.0,
              minWidth: 480,
              minHeight: 640,
            );
            final imageUintList = await FlutterImageCompress.compressWithList(
              bytes,
              minWidth: originalImage.width ~/ scale,
              minHeight: originalImage.height ~/ scale,
            );
            stopTimer();
            pop(result: base64Encode(imageUintList));
            // widget.onEvent.call(base64Encode(imageUintList));
          } else {
            isTakePictureProcess = false;
            stopTimer();
          }
        } else {
          isTakePictureProcess = false;
          stopTimer();
        }
      }
    }
    if (!mounted) setState(() {});
  }

  void startTimer() {
    if (_timer?.isActive == false || _timer == null) {
      _timer = Timer.periodic(const Duration(seconds: 1), _onTimerTick);
    }
  }

  void stopTimer() {
    if (_timer?.isActive == true) {
      _timer?.cancel();
      counter = 3;
    }
  }

  @override
  void dispose() {
    _stopLiveFeed();
    super.dispose();
  }

  Future _stopLiveFeed() async {
    await _controller?.stopImageStream();
    await _controller?.dispose();
    _controller = null;
  }
}
